package com.persagy.apm.energy.report.common.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

public class DateUtils {
    public static final String SDFSECOND = "yyyyMMddHHmmss";
    public static final String SDF_SECOND = "yyyy-MM-dd HH:mm:ss";

    /**
     * 将pattern格式的字符串日期转换为Date类型日期
     *
     * @param source  pattern格式的字符串日期
     * @param pattern 日期格式如yyyy-MM-dd HH:mm:ss
     * @return
     * @throws Exception
     */
    public static Date str2Date(String source, String pattern) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        return sdf.parse(source);
    }

    /**
     * 将Date类型的日期转换成pattern格式的字符串日期
     *
     * @param date
     * @param pattern
     * @return
     * @throws Exception
     */
    public static String date2Str(Date date, String pattern) {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        return sdf.format(date);
    }

    /**
     * 获取时间按年偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return
     */
    public static Date getYearOff(Date date, int offset) {
        return getOffset(date, offset, 5);
    }

    /**
     * 获取时间按月偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return
     */
    public static Date getMonthOff(Date date, int offset) {
        return getOffset(date, offset, 4);
    }

    /**
     * 获取时间按周偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return
     */
    public static Date getWeekOff(Date date, int offset) {
        return getOffset(date, offset, 3);
    }

    /**
     * 获取时间按日偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return
     */
    public static Date getDayOff(Date date, int offset) {
        return getOffset(date, offset, 2);
    }

    /**
     * 获取时间按小时偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return
     */
    public static Date getHourOff(Date date, int offset) {
        return getOffset(date, offset, 1);
    }

    /**
     * description 获取时间按分钟偏移offset个单位后的时间
     *
     * @param date
     * @param offset
     * @return java.util.Date
     * @author feng
     * @since 18:51 2018/12/10
     */
    public static Date getMinuteOff(Date date, int offset) {
        return getOffset(date, offset, 0);
    }

    /**
     * 获取指定时间粒度偏移量偏移后的时间
     *
     * @param date
     * @param offset int 偏移量
     * @param type   int 时间粒度0-分，1-时，2-天，3-周，4-月，5-年
     * @return
     */
    private static Date getOffset(Date date, int offset, int type) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        switch (type) {
            case 0:
                cal.add(Calendar.MINUTE, offset);
                break;
            case 1:
                cal.add(Calendar.HOUR_OF_DAY, offset);
                break;
            case 2:
                cal.add(Calendar.DAY_OF_MONTH, offset);
                break;
            case 3:
                cal.add(Calendar.DAY_OF_MONTH, offset * 7);
                break;
            case 4:
                cal.add(Calendar.MONTH, offset);
                break;
            case 5:
                cal.add(Calendar.YEAR, offset);
                break;
            default:
                break;
        }
        return cal.getTime();
    }

    /**
     * 获取某一天的开始时间 即2017-01-01 00:00:00
     *
     * @param date 时间参数
     * @return
     * @throws ParseException
     */
    public static Date getStartTimeOfDay(Date date) throws ParseException {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int year = c.get(Calendar.YEAR);
        int month = c.get(Calendar.MONTH);
        int day = c.get(Calendar.DAY_OF_MONTH);
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        c.set(Calendar.MILLISECOND, 0);
        c.set(Calendar.DAY_OF_MONTH, day);
        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, month);
        return c.getTime();
    }

    public static Date getFirstDayOfYear(Date date) {
        Calendar c = Calendar.getInstance();
        c.setTime(date);
        int year = c.get(Calendar.YEAR);
        c.set(Calendar.HOUR_OF_DAY, 0);
        c.set(Calendar.DAY_OF_MONTH, 1);
        c.set(Calendar.MINUTE, 0);
        c.set(Calendar.SECOND, 0);
        c.set(Calendar.MILLISECOND, 0);
        c.set(Calendar.YEAR, year);
        c.set(Calendar.MONTH, 0);
        return c.getTime();
    }

}
