package com.persagy.apm.energy.report.workorder.service.impl;

import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.common.dto.QueryAreaPlatformParamDTO;
import com.persagy.apm.energy.report.common.dto.QueryPjPlatformParamDTO;
import com.persagy.apm.energy.report.common.service.CommonService;
import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.monthly.detail.business.model.Platform;
import com.persagy.apm.energy.report.workorder.client.WorkOrderWebClient;
import com.persagy.apm.energy.report.workorder.model.dto.QueryWorkOrderListByParamDTO;
import com.persagy.apm.energy.report.workorder.service.IWorkOrderWebService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TreeMap;
import java.util.stream.Collectors;


@Service
public class IWorkOrderWebServiceImpl implements IWorkOrderWebService {

    @Resource
    private WorkOrderWebClient workOrderWebClient;
    @Resource
    private CommonService commonService;

    @Override
    public void getPjWorkOrderRate(QueryPjPlatformParamDTO queryPjPlatformParamDTO, Platform platform) {
        TreeMap<Date, Double> date_rateMap = new TreeMap<>();
        Date reportDate = queryPjPlatformParamDTO.getReportDate();
        Date startDate = DateUtils.getFirstDayOfYear(reportDate);
        while (startDate.before(DateUtils.getMonthOff(reportDate, 1))) {
            date_rateMap.put(startDate, getWorkOrderCompleteRate(queryPjPlatformParamDTO.getProjectId(), startDate, queryPjPlatformParamDTO.getUserId()));
            startDate = DateUtils.getMonthOff(startDate, 1);
        }
        List<Double> monthlySummaries = date_rateMap.values().stream().collect(Collectors.toList());
        platform.setMonthlySummaries(monthlySummaries);
    }

    private Double getWorkOrderCompleteRate(String projectId, Date reportDate, String userId) {
        Double completeRate = null;
        try {
            QueryWorkOrderListByParamDTO paramDTO = new QueryWorkOrderListByParamDTO();
            paramDTO.setUser_id(userId);
            paramDTO.setProject_id(projectId);
            paramDTO.setTimeType("ask_end_time");
            paramDTO.setStart_time(DateUtils.date2Str(reportDate, DateUtils.SDFSECOND));
            paramDTO.setEnd_time(DateUtils.date2Str(DateUtils.getMonthOff(reportDate, 1), DateUtils.SDFSECOND));
            PoemsFeignResponse<Object> response = workOrderWebClient.listWorkOrderByParam(paramDTO);
            Integer dealCount = response.getCount();
            paramDTO.setTimeType("create_time");
            response = workOrderWebClient.listWorkOrderByParam(paramDTO);
            Integer workCount = response.getCount();
            if (null != workCount && workCount != 0) {
                completeRate = Double.valueOf(dealCount) / workCount * 100;
            }
            return completeRate;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return completeRate;
    }

    @Override
    public void getAreaWorkOrderRate(QueryAreaPlatformParamDTO queryAreaPlatformParamDTO, Platform platform) {
        // 合格标准
        Double standardValue = platform.getStandardValue();
        // 校验规则
        String qualifyFormula = platform.getQualifyFormula();
        // 达标数量
        Integer satisfiedCount = 0;
        // 未达标数量
        Integer unsatisfiedCount = 0;
        // 未达标项目id列表
        List<String> unsatisfiedPjIdList = new ArrayList<>();
        for (String pjId : queryAreaPlatformParamDTO.getProjectIdList()) {
            Double completeRate = getWorkOrderCompleteRate(pjId, queryAreaPlatformParamDTO.getReportDate(), queryAreaPlatformParamDTO.getUserId());
            if (null != completeRate && null != standardValue) {
                if (DataUtils.getQualifyResult(standardValue, qualifyFormula, completeRate) == 0.0) {
                    unsatisfiedCount++;
                    unsatisfiedPjIdList.add(pjId);
                } else {
                    satisfiedCount++;
                }
            }
        }
        // 根据未达标项目id列表获取不达标项目名称
        List<String> pjNameList = commonService.getPjNameByPjId(unsatisfiedPjIdList);
        platform.setSatisfiedCount(satisfiedCount);
        platform.setUnsatisfiedCount(unsatisfiedCount);
        platform.setUnsatisfiedProjects(pjNameList);
    }

}
